package provide cgi

proc cgi_unquote_input {buf} {
	# rewrite "+" back to space
	regsub -all {\+} $buf { } buf
	# protect \ from quoting another \ and throwing off other things first
	# protect $ from doing variable expansion
	# protect [ from doing evaluation
	# protect " from terminating string
	regsub -all {([\\["$])} $buf {\\\1} buf
	
	# replace line delimiters with newlines
	regsub -all -nocase "%0d%0a" $buf "\n" buf
	# Mosaic sends just %0A.  This is handled in the next command.

	# prepare to process all %-escapes 
	regsub -all -nocase {%([a-f0-9][a-f0-9])} $buf {[format %c 0x\1]} buf
	# process %-escapes and undo all protection
	eval return \"$buf\"
}

# return string but with html-special characters escaped,
# necessary if you want to send unknown text to an html-formatted page.
proc cgi_quote_html {s} {
    regsub -all {&}	$s {\&amp;}	s	;# must be first!
    regsub -all {"}	$s {\&quot;}	s
    regsub -all {<}	$s {\&lt;}	s
    regsub -all {>}	$s {\&gt;}	s
    return $s
}

# return string quoted appropriately to appear in a url
proc cgi_quote_url {in} {
    regsub -all {%}  $in "%25" in
    regsub -all {\+} $in "%2b" in
    regsub -all { }  $in "%20" in
    regsub -all {"}  $in "%22" in
    regsub -all {\?} $in "%3f" in
    regsub -all {&} $in "%26" in
    return $in
}

proc cgi_input {{raw 0}} {
	global env
	global _cgi

	set _cgi [dict create]

	set _cgi(method) "GET"
	if {[info exists env(REQUEST_METHOD)]} {
		set _cgi(method) $env(REQUEST_METHOD)
	}

	set input ""
	switch $_cgi(method) {
		"GET" {
			catch {set input $env(QUERY_STRING)} # Optional
		}
		"POST" {
			if {[info exists env(CONTENT_LENGTH)]} {
				set input [read stdin $env(CONTENT_LENGTH)]
			}
		}
	}

	#puts "Input ($input)<br>"
	#puts "Input length: [string length $input]<br>"
	set pairs [split $input &]
	foreach pair $pairs {
		#puts "Pair ($pair)<br>"
		if {0 == [regexp {^([^=]*)=(.*)$} $pair dummy varname val]} {
			set varname anonymous
			set val $pair
		}
		if {$raw} {
			set varname [cgi_unquote_input $varname]
			set val $val
		} else {
			set varname [cgi_unquote_input $varname]
			set val [cgi_unquote_input $val]
		}

		#puts "VAR: ($varname), VAL: ($val)<br>"

		# Handle array submissions
		if {[string range $varname end-1 end] eq "\[\]"} {
			set varname [string range $varname 0 end-2]
			lappend _cgi($varname) $val
		} else {
			dict set _cgi $varname $val
			#puts "Set $varname = $val<br>"
		}
	}
}

proc cgi_dump {} {
	global _cgi

	foreach name [dict keys $_cgi] {
		puts "CGI: $name = [dict get $_cgi $name]<br>"
	}
}

proc cgi_get {var {def 0}} {
	global _cgi

	if {![info exists _cgi]} cgi_input

	if {[dict exists $_cgi $var]} {
		return $_cgi($var)
	} else {
		return $def
	}
}

proc cgi_exists {var} {
	global _cgi

	if {![info exists _cgi]} cgi_input

	return [dict exists $_cgi $var]
}

